/* Copyright (C) 1998-9 Pancrazio `Ezio' de Mauro <p@demauro.net>
 * Copyright (C) 2004 Milosz `Dem' Tanski <mtanski@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: initialize.c,v 1.2 2004/12/20 01:36:35 demie Exp $
 *
 * -----------------------------------------------------------------
 * initialize.c: library initalization hooks
 */

#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>

#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <syslog.h>
#include <errno.h>
#include <unistd.h>
#include <string.h>
#include <time.h>

#include <fcntl.h>
#include <dlfcn.h>

#include "zengarden.h"

int initialize = 0;
/* Some allocated strings that are paths */
char *instroot = 0;
char *logfile = 0;
char *bpath = 0;
char *bjail = 0;
/* hope we can speed things up by "caching" the len */
int irlen = 0;
int bplen = 0;
int bjlen = 0;

void init(void) {
  char *tmp;

  /* ZG_ROOT_PATH */
  if ((tmp = getenv("ZG_ROOT_PATH"))) {
    if (!(instroot = strdup(tmp))) {
      syslog(LOGLEVEL, "%i: Could not allocate enouth memory: %s\n", (__LINE__-1), strerror(errno));
      _exit(EXIT_FAILURE);
    }
    irlen = strlen(instroot);
  }
  /* ZG_BUILD_PATH */
  if ((tmp = getenv("ZG_BUILD_PATH"))) {
    if (!(bjail = strdup(tmp))) {
      syslog(LOGLEVEL, "%i: Could not allocate enouth memory: %s\n", (__LINE__-1), strerror(errno));
      _exit(EXIT_FAILURE);
    }
    bjlen = strlen(bjail);
  }
  /* ZG_LOG_FILE */
  if ((tmp = getenv("ZG_LOG_FILE"))) {
    logfile = strdup(tmp);
    if (!logfile) {
      syslog(LOGLEVEL, "%i: Could not allocate enouth memory: %s\n", (__LINE__-1), strerror(errno));
      _exit(EXIT_FAILURE);
    }
  }
  /* ZG_BACKUP_PATH */
  if ((tmp = getenv("ZG_BACKUP_PATH"))) {
    if (!(bpath = strdup(tmp))) {
      syslog(LOGLEVEL, "%i: Could not allocate enouth memory: %s\n", (__LINE__-1), strerror(errno));
      _exit(EXIT_FAILURE);
    }
    bplen = strlen(bpath);
  }

  /* The can't use multiple features at the same time */
  if ((instroot && bpath) || (instroot && bjail) || (bpath && bjail)) {
    syslog(LOGLEVEL, "%i: Multiple featues specified\n", (__LINE__-1));
    _exit(EXIT_FAILURE);
  }

  initialize = 1;
}

void _init(void) {
  /* save the memory location of the real glibc functions */
  true_time = dlsym(RTLD_NEXT, "time");
  true_chmod = dlsym(RTLD_NEXT, "chmod");
  true_chown = dlsym(RTLD_NEXT, "chown");
  true_chroot = dlsym(RTLD_NEXT, "chroot");
  true_creat = dlsym(RTLD_NEXT, "creat");
  true_fchmod = dlsym(RTLD_NEXT, "fchmod");
  true_fchown = dlsym(RTLD_NEXT, "fchown");
  true_fopen = dlsym(RTLD_NEXT, "fopen");
  true_ftruncate = dlsym(RTLD_NEXT, "ftruncate");
  true_lchown = dlsym(RTLD_NEXT, "lchown");
  true_link = dlsym(RTLD_NEXT, "link");
  true_mkdir = dlsym(RTLD_NEXT, "mkdir");
  true_open = dlsym(RTLD_NEXT, "open");
  true_rename = dlsym(RTLD_NEXT, "rename");
  true_rmdir = dlsym(RTLD_NEXT, "rmdir");
  true_symlink = dlsym(RTLD_NEXT, "symlink");
  true_truncate = dlsym(RTLD_NEXT, "truncate");
  true_unlink = dlsym(RTLD_NEXT, "unlink");
  // 64 bit
  true_creat64 = dlsym(RTLD_NEXT, "creat64");
  true_fopen64 = dlsym(RTLD_NEXT, "fopen64");
  true_ftruncate64 = dlsym(RTLD_NEXT, "ftruncate64");
  true_open64 = dlsym(RTLD_NEXT, "open64");
  true_truncate64 = dlsym(RTLD_NEXT, "truncate64");
}

void _fini(void) {
  /* clean up */
  if (instroot)
    free(instroot);
  if (logfile)
    free(logfile);
  if (bpath)
    free(bpath); 
}

// vim:tabstop=2:shiftwidth=2:expandtab:
